package demo_test

import (
	"demo"
	"runtime"
	"strings"
	"testing"
)

type MockDoPrint struct {
	t               *testing.T
	doCallsCount    int
	endDoCallsCount int
}

func (d *MockDoPrint) Do() {
	_, file, _, _ := runtime.Caller(1)
	if strings.Contains(file, "_test.go") {
		d.t.Fatal("Do() should not be called in tests")
	}
	d.doCallsCount++
}

func (d *MockDoPrint) EndDo() {
	_, file, _, _ := runtime.Caller(1)
	if strings.Contains(file, "_test.go") {
		d.t.Fatal("EndDo() should not be called in tests")
	}
	d.endDoCallsCount++
}

func TestRun(t *testing.T) {
	mockDoPrint := &MockDoPrint{
		t: t,
	}
	demo := demo.MyStruct{
		D: mockDoPrint,
	}
	demo.Run()
	demo.D.EndDo()

	t.Run("Do() call count", func(t *testing.T) {
		want := 1
		got := mockDoPrint.doCallsCount
		if got != want {
			t.Fatalf("Do() call count = %d; want %d", got, want)
		}
	})

	t.Run("EndDo() call count", func(t *testing.T) {
		want := 1
		got := mockDoPrint.endDoCallsCount
		if got != want {
			t.Fatalf("EndDo() call count = %d; want %d", got, want)
		}
	})
}
